; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 17 - Sine Wave Generator
; Link: https://www.circuitbread.com/tutorials/sine-wave-generator-part-17-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
PSA         EQU 0003h
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h

PS0         EQU 0000h
PS1         EQU 0001h
PS2         EQU 0002h

;----- STATUS Bits -----------------------------------------------------
C           EQU 0000h
Z           EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)


; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW  ~(1<<T0CS)   ;Configure GP2 as GPIO
    OPTION
    CLRW                ;Clear the TRISGPIO register to configure
    TRIS GPIO           ;all GPIOs as outputs

;Preload the registers with the required values according to the sine table
    MOVLW 4
    MOVWF 0x10
    MOVWF 0x18
    MOVLW 5
    MOVWF 0x11
    MOVWF 0x17
    MOVLW 6
    MOVWF 0x12
    MOVWF 0x16
    MOVLW 7
    MOVWF 0x13
    MOVWF 0x14
    MOVWF 0x15
    MOVLW 2
    MOVWF 0x19
    MOVWF 0x1F
    MOVLW 1
    MOVWF 0x1A
    MOVWF 0x1E
    CLRF 0x1B
    CLRF 0x1C
    CLRF 0x1D

    MOVLW 0x10          ;Copy the 0x10 register address to the
    MOVWF FSR           ;indirect address pointer

LOOP:                   ;Start of the main loop
    MOVF INDF, W        ;Read the value of the indirectly addressed
    MOVWF GPIO          ;register and copy it into the GPIO
    INCF FSR            ;Set the next address to read
    BTFSS FSR, 4        ;If bit 4 of the FSR register is not set
    BSF FSR, 4          ;then set it
    GOTO LOOP           ;loop forever

    END INIT        ; Program entry point.